* File Name:  BrowPick.Prg
*  Function:  Use Browse command to select a record from a dbf
*   Returns:  An expression specified by the calling program (or recno())
*  Language:  FoxPro 1.02
*   Created:  Sometime in 1990, Maurice L. Frank

*     Notes:  This is designed to be called by validation programs
*             It is not idiot proof; there is virtually no error checking
*             and many assumptions are made
*             All parameters are optional - see defaults listed below

*             An earlier version of this file was demonstrated at the 
*             ADBUS FOX SIG meeting Thursday May 16, 1991
*             This version adds four enhancements for incremental searching:
*             1.  the backspace key now chops off the last key typed
*                 (backspace is 127 NOT 27 as I had it before)
*             2.  there is now a time delay of 3 seconds
*                 waiting more than 3 seconds begins a new search string
*                 this is similar to how FoxPro help works
*             3.  a space character can be used in the search string (i.e. New York)
*             4.  the original record (before seeking search string) is
*                 limited to the last record in file (reccount)


parameters mdbf, midx, mret_expr, mtitle, mtop, mleft, mbottom, mright


* parameter definitions and default if null parameter

* parameter    meaning                 default
* ---------    ---------------------   ------------------------------
* mdbf         dbf name                alias() (currently open dbf)
* midx         idx file, list, or ""   "" (whatever is active, or none)
* mret_expr    return expression       recno()
* mtitle       window title            alias() (of mdbf)
* mtop         top row of window       01 (leave room for scoreboard)
* mleft        left corner             00
* mbottom      bottom row of window    srows()-1 (24 on 80x25 screen)
* mright       right corner            scols()-1 (79 on 80x25 screen)


private all


* some memvars we need

mlastrow = srows()-1                               && last row of screen
mlastcol = scols()-1                               && last column of screen

* process default parameters

if !empty(mdbf)
   * open dbf requested by caller; if none requested, assume one is open and use it
   select 0
   use (mdbf)
endif

if !empty(midx)
   * midx may be one or more (a comma delimited list); can't use () on a list
   set index to &midx
endif

morder = sys(21)                                   && "0" if no index active

mtitle = iif(empty(mtitle),alias(),mtitle)

* since this is a function, it must return something
* if calling program doesn't want anything, send recno()
* note that the caller can pass in a compound expression: "st+city"
* also note that the caller must handle the data typing received

mret_expr = iif(empty(mret_expr),"RECNO()",upper(mret_expr))

* window coordinates: default is full screen - this can be used for inquiry

mtop = iif(empty(mtop),01,mtop)
mleft = iif(empty(mleft),00,mleft)
mbottom = iif(empty(mbottom),mlastrow-1,mbottom)
mright = iif(empty(mright),mlastcol,mright)


* define windows: one to browse in; one for help messages

define window BROWPICK from mtop,mleft to mbottom,mright none color scheme 10
define window BP_MSG from mlastrow,00 to mlastrow,mlastcol none

activate window BP_MSG
?  iif(parameters()>=3,"Enter = select   ","") + "ESCape = cancel"

* activate hot keys using on key label
* (F2 is a safety valve to cancel all hot keys including ENTER)

on key label F2 on key

* enter key selects a record - ret_expr applies to that record
* we stuff keyboard with ascii value of ctrl-end/ctrl-w to terminate browse

on key label ENTER keyboard chr(23)

if morder <> "0"
   * an index is active - we can search data base
   ?? "  A-Z, 0-9, Space, Backspace = search"
   
   * set up hot keys for incremental search - all letters and digits
   * (sorry for the macros here)

   * alpha chars
   for i = 1 to 26
      key = chr(64+i)
      on key label &key do bp_srch
   next

   * digits 0 - 9
   for i = 0 to 9
      key = chr(48+i)
      on key label &key do bp_srch
   next

   on key label spacebar do bp_srch
   on key label backspace do bp_srch

   * initialize string to hold incremental search value

   msrch_val = ""
   msrch_time = seconds()
   mreccount = reccount()

endif


* 120+ lines later, here is the browse

browse TITLE mtitle WINDOW browpick nomodify noappend nodelete


* disable hot keys as soon as user is out of browse

on key label ENTER

if morder <> "0"
   for i = 1 to 26
      key = chr(64+i)
      on key label &key
   next
   for i = 0 to 9
      key = chr(48+i)
      on key label &key
   next
   on key label spacebar
   on key label backspace
endif

on key label F2


* store key user hit to get out of browse

mlastkey = lastkey()

* let's figure out what to send back via the return statement

if !empty(mret_expr)
   * calling program wants something returned
   if mlastkey <> 27
      * escape not pressed (probably enter or ctrl-end), return something
      mret_val = &mret_expr
   else
      * ESCape! go the the infamous "phanthom record" and return blanks
      * calling program must check for empty return values
      if mret_expr <> "RECNO()"
         go bottom
         skip
         mret_val = &mret_expr
      else
         mret_val = -1
      endif
   endif
else
   * no explicit request for a return value; a function must return something
   mret_val = iif(mlastkey<>27,recno(),0)
endif

if !empty(mdbf)
   * close the dbf if this program opened it, otherwise, leave it open
   * we are not restoring previous index if different from parameter received
   use
endif

* shut the windows

release window browpick, bp_msg

* let's get out of here!

return mret_val




PROCEDURE BP_SRCH

* incremental search while in a browse window; an index must be active

* set step on


if lastkey() <> 127
   * save our place in case seek fails
   morig_rec = min(recno(),mreccount)
   
   if seconds() - msrch_time < 3
      * add this character to search string; force it to uppercase
      msrch_val = msrch_val + upper(chr(lastkey()))
   else
      * start a new search string - too much time elapsed between keys
      msrch_val = upper(chr(lastkey()))
   endif
   msrch_time = seconds()
   
   seek msrch_val
   if !found()
      ?? chr(07)
      * return to where we started from
      goto morig_rec
      * strip bad character off search string
      msrch_val = left(msrch_val,len(msrch_val)-1)
   endif
else
   * backspace (chr 127)
   if !empty(msrch_val)
      * remove last character in string
      msrch_val = left(msrch_val,len(msrch_val)-1)
      seek msrch_val                           && assume found
   endif
endif

* if seek was succesful, record pointer is now at first matching record

*set step off

return

